@@ -11,6 +11,8 @@ mod progress;
 mod utils;
 
 use crate::cli::Cli;
+use crate::config::Config;
+use crate::display_node::DisplayNode;
 use crate::progress::RuntimeErrors;
 use clap::Parser;
 use dir_walker::WalkData;
@@ -20,13 +22,14 @@ use progress::PIndicator;
 use regex::Error;
 use std::collections::HashSet;
 use std::env;
-use std::fs::read_to_string;
+use std::fs::{read, read_to_string};
 use std::io;
+use std::io::Read;
 use std::panic;
 use std::process;
 use std::sync::Arc;
 use std::sync::Mutex;
-use sysinfo::{System, SystemExt};
+use sysinfo::System;
 use utils::canonicalize_absolute_path;
 
 use self::display::draw_it;
@@ -66,7 +69,7 @@ fn should_init_color(no_color: bool, force_color: bool) -> bool {
     {
         // Required for windows 10
         // Fails to resolve for windows 8 so disable color
-        match ansi_term::enable_ansi_support() {
+        match nu_ansi_term::enable_ansi_support() {
             Ok(_) => true,
             Err(_) => {
                 eprintln!("This version of Windows does not support ANSI colors");
@@ -125,34 +128,15 @@ fn main() {
     })
     .expect("Error setting Ctrl-C handler");
 
-    let target_dirs = match config.get_files_from(&options) {
-        Some(path) => {
-            if path == "-" {
-                let mut targets_to_add = io::stdin()
-                    .lines()
-                    .map_while(Result::ok)
-                    .collect::<Vec<String>>();
-
-                if targets_to_add.is_empty() {
-                    eprintln!("No input provided, defaulting to current directory");
-                    targets_to_add.push(".".to_owned());
-                }
-                targets_to_add
-            } else {
-                // read file
-                match read_to_string(path) {
-                    Ok(file_content) => file_content.lines().map(|x| x.to_string()).collect(),
-                    Err(e) => {
-                        eprintln!("Error reading file: {e}");
-                        vec![".".to_owned()]
-                    }
-                }
-            }
-        }
-        None => match options.params {
+    let target_dirs = if let Some(path) = config.get_files0_from(&options) {
+        read_paths_from_source(&path, true)
+    } else if let Some(path) = config.get_files_from(&options) {
+        read_paths_from_source(&path, false)
+    } else {
+        match options.params {
             Some(ref values) => values.clone(),
             None => vec![".".to_owned()],
-        },
+        }
     };
 
     let summarize_file_types = options.file_types;
@@ -170,7 +154,7 @@ fn main() {
     // If depth is set, then we set the default number_of_lines to be max
     // instead of screen height
 
-    let number_of_lines = match options.number_of_lines {
+    let number_of_lines = match config.get_number_of_lines(&options) {
         Some(val) => val,
         None => {
             if depth != usize::MAX {
@@ -218,9 +202,12 @@ fn main() {
     let limit_filesystem = options.limit_filesystem;
     let follow_links = options.dereference_links;
 
-    let allowed_filesystems = limit_filesystem
-        .then(|| get_filesystem_devices(&target_dirs, follow_links))
-        .unwrap_or_default();
+    let allowed_filesystems = if limit_filesystem {
+        get_filesystem_devices(&target_dirs, follow_links)
+    } else {
+        Default::default()
+    };
+
     let simplified_dirs = simplify_dir_names(&target_dirs);
 
     let ignored_full_path: HashSet<PathBuf> = ignore_directories
@@ -237,7 +224,7 @@ fn main() {
         indicator.spawn(output_format.clone())
     }
 
-    let keep_collapsed: HashSet<PathBuf> = match options.collapse {
+    let keep_collapsed: HashSet<PathBuf> = match config.get_collapse(&options) {
         Some(ref collapse) => {
             let mut combined_dirs = HashSet::new();
             for collapse_dir in collapse {
@@ -270,32 +257,98 @@ fn main() {
         progress_data: indicator.data.clone(),
         errors: errors_for_rayon,
     };
+
     let threads_to_use = config.get_threads(&options);
     let stack_size = config.get_custom_stack_size(&options);
-    init_rayon(&stack_size, &threads_to_use);
 
-    let top_level_nodes = walk_it(simplified_dirs, &walk_data);
+    init_rayon(&stack_size, &threads_to_use).install(|| {
+        let top_level_nodes = walk_it(simplified_dirs, &walk_data);
 
-    let tree = match summarize_file_types {
-        true => get_all_file_types(&top_level_nodes, number_of_lines, &by_filetime),
-        false => {
-            let agg_data = AggregateData {
-                min_size: config.get_min_size(&options),
-                only_dir: config.get_only_dir(&options),
-                only_file: config.get_only_file(&options),
-                number_of_lines,
-                depth,
-                using_a_filter: !filter_regexs.is_empty() || !invert_filter_regexs.is_empty(),
-                short_paths: !config.get_full_paths(&options),
-            };
-            get_biggest(top_level_nodes, agg_data, &by_filetime, keep_collapsed)
+        let tree = match summarize_file_types {
+            true => get_all_file_types(&top_level_nodes, number_of_lines, walk_data.by_filetime),
+            false => {
+                let agg_data = AggregateData {
+                    min_size: config.get_min_size(&options),
+                    only_dir: config.get_only_dir(&options),
+                    only_file: config.get_only_file(&options),
+                    number_of_lines,
+                    depth,
+                    using_a_filter: !filter_regexs.is_empty() || !invert_filter_regexs.is_empty(),
+                    short_paths: !config.get_full_paths(&options),
+                };
+                get_biggest(
+                    top_level_nodes,
+                    agg_data,
+                    walk_data.by_filetime,
+                    keep_collapsed,
+                )
+            }
+        };
+
+        // Must have stopped indicator before we print to stderr
+        indicator.stop();
+
+        let print_errors = config.get_print_errors(&options);
+        let final_errors = walk_data.errors.lock().unwrap();
+        print_any_errors(print_errors, &final_errors);
+
+        if tree.children.is_empty() && !final_errors.file_not_found.is_empty() {
+            std::process::exit(1)
+        } else {
+            print_output(
+                config,
+                options,
+                tree,
+                walk_data.by_filecount,
+                is_colors,
+                terminal_width,
+            )
         }
-    };
+    });
+}
 
-    // Must have stopped indicator before we print to stderr
-    indicator.stop();
+fn print_output(
+    config: Config,
+    options: Cli,
+    tree: DisplayNode,
+    by_filecount: bool,
+    is_colors: bool,
+    terminal_width: usize,
+) {
+    let output_format = config.get_output_format(&options);
 
-    let final_errors = walk_data.errors.lock().unwrap();
+    if config.get_output_json(&options) {
+        OUTPUT_TYPE.with(|wrapped| {
+            if by_filecount {
+                wrapped.replace("count".to_string());
+            } else {
+                wrapped.replace(output_format);
+            }
+        });
+        println!("{}", serde_json::to_string(&tree).unwrap());
+    } else {
+        let idd = InitialDisplayData {
+            short_paths: !config.get_full_paths(&options),
+            is_reversed: !config.get_reverse(&options),
+            colors_on: is_colors,
+            by_filecount,
+            by_filetime: config.get_filetime(&options),
+            is_screen_reader: config.get_screen_reader(&options),
+            output_format,
+            bars_on_right: config.get_bars_on_right(&options),
+        };
+
+        draw_it(
+            idd,
+            &tree,
+            config.get_no_bars(&options),
+            terminal_width,
+            config.get_skip_total(&options),
+        )
+    }
+}
+
+fn print_any_errors(print_errors: bool, final_errors: &RuntimeErrors) {
     if !final_errors.file_not_found.is_empty() {
         let err = final_errors
             .file_not_found
@@ -303,17 +356,17 @@ fn main() {
             .map(|a| a.as_ref())
             .collect::<Vec<&str>>()
             .join(", ");
-        eprintln!("No such file or directory: {}", err);
+        eprintln!("No such file or directory: {err}");
     }
     if !final_errors.no_permissions.is_empty() {
-        if config.get_print_errors(&options) {
+        if print_errors {
             let err = final_errors
                 .no_permissions
                 .iter()
                 .map(|a| a.as_ref())
                 .collect::<Vec<&str>>()
                 .join(", ");
-            eprintln!("Did not have permissions for directories: {}", err);
+            eprintln!("Did not have permissions for directories: {err}");
         } else {
             eprintln!(
                 "Did not have permissions for all directories (add --print-errors to see errors)"
@@ -327,76 +380,105 @@ fn main() {
             .map(|a| a.as_ref())
             .collect::<Vec<&str>>()
             .join(", ");
-        eprintln!("Unknown Error: {}", err);
+        eprintln!("Unknown Error: {err}");
     }
+}
 
-    if let Some(root_node) = tree {
-        if config.get_output_json(&options) {
-            OUTPUT_TYPE.with(|wrapped| {
-                wrapped.replace(output_format);
-            });
-            println!("{}", serde_json::to_string(&root_node).unwrap());
+fn read_paths_from_source(path: &str, null_terminated: bool) -> Vec<String> {
+    let from_stdin = path == "-";
+
+    let result: Result<Vec<String>, Option<String>> = (|| {
+        // 1) read bytes
+        let bytes = if from_stdin {
+            let mut b = Vec::new();
+            io::stdin().lock().read_to_end(&mut b).map_err(|_| None)?;
+            b
         } else {
-            let idd = InitialDisplayData {
-                short_paths: !config.get_full_paths(&options),
-                is_reversed: !config.get_reverse(&options),
-                colors_on: is_colors,
-                by_filecount,
-                by_filetime,
-                is_screen_reader: config.get_screen_reader(&options),
-                output_format,
-                bars_on_right: config.get_bars_on_right(&options),
-            };
+            read(path).map_err(|e| Some(e.to_string()))?
+        };
 
-            draw_it(
-                idd,
-                config.get_no_bars(&options),
-                terminal_width,
-                &root_node,
-                config.get_skip_total(&options),
-            )
+        let text = std::str::from_utf8(&bytes).map_err(|e| {
+            if from_stdin {
+                None
+            } else {
+                Some(e.to_string())
+            }
+        })?;
+        let items: Vec<String> = if null_terminated {
+            text.split('\0')
+                .filter(|s| !s.is_empty())
+                .map(str::to_owned)
+                .collect()
+        } else {
+            text.lines().map(str::to_owned).collect()
+        };
+        if from_stdin && items.is_empty() {
+            return Err(None);
+        }
+        Ok(items)
+    })();
+
+    match result {
+        Ok(v) => v,
+        Err(None) => {
+            eprintln!("No files provided, defaulting to current directory");
+            vec![".".to_owned()]
+        }
+        Err(Some(msg)) => {
+            eprintln!("Failed to read file: {msg}");
+            vec![".".to_owned()]
         }
     }
 }
 
-fn init_rayon(stack_size: &Option<usize>, threads: &Option<usize>) {
-    // Rayon seems to raise this error on 32-bit builds
-    // The global thread pool has not been initialized.: ThreadPoolBuildError { kind: GlobalPoolAlreadyInitialized }
-    if cfg!(target_pointer_width = "64") {
-        let result = panic::catch_unwind(|| build_thread_pool(*stack_size, *threads));
-        if result.is_err() {
-            eprintln!("Problem initializing rayon, try: export RAYON_NUM_THREADS=1")
+fn init_rayon(stack: &Option<usize>, threads: &Option<usize>) -> rayon::ThreadPool {
+    let stack_size = match stack {
+        Some(s) => Some(*s),
+        None => {
+            // Do not increase the stack size on a 32 bit system, it will fail
+            if cfg!(target_pointer_width = "32") {
+                None
+            } else {
+                let large_stack = usize::pow(1024, 3);
+                let mut sys = System::new_all();
+                sys.refresh_memory();
+                // Larger stack size if possible to handle cases with lots of nested directories
+                let available = sys.available_memory();
+                if available > (large_stack * threads.unwrap_or(1)).try_into().unwrap() {
+                    Some(large_stack)
+                } else {
+                    None
+                }
+            }
+        }
+    };
+
+    match build_thread_pool(stack_size, threads) {
+        Ok(pool) => pool,
+        Err(err) => {
+            eprintln!("Problem initializing rayon, try: export RAYON_NUM_THREADS=1");
+            if stack.is_none() && stack_size.is_some() {
+                // stack parameter was none, try with default stack size
+                if let Ok(pool) = build_thread_pool(None, threads) {
+                    eprintln!("WARNING: not using large stack size, got error: {err}");
+                    return pool;
+                }
+            }
+            panic!("{err}");
         }
     }
 }
 
 fn build_thread_pool(
-    stack: Option<usize>,
-    threads: Option<usize>,
-) -> Result<(), rayon::ThreadPoolBuildError> {
-    let mut pool = rayon::ThreadPoolBuilder::new();
-
-    if let Some(thread_count) = threads {
-        pool = pool.num_threads(thread_count);
-    }
-
-    let stack_size = match stack {
-        Some(s) => Some(s),
-        None => {
-            let large_stack = usize::pow(1024, 3);
-            let mut s = System::new();
-            s.refresh_memory();
-            // Larger stack size if possible to handle cases with lots of nested directories
-            let available = s.available_memory();
-            if available > large_stack.try_into().unwrap() {
-                Some(large_stack)
-            } else {
-                None
-            }
-        }
-    };
+    stack_size: Option<usize>,
+    threads: &Option<usize>,
+) -> Result<rayon::ThreadPool, rayon::ThreadPoolBuildError> {
+    let mut pool_builder = rayon::ThreadPoolBuilder::new();
     if let Some(stack_size_param) = stack_size {
-        pool = pool.stack_size(stack_size_param);
+        pool_builder = pool_builder.stack_size(stack_size_param);
     }
-    pool.build_global()
+    if let Some(thread_count) = threads {
+        pool_builder = pool_builder.num_threads(*thread_count);
+    }
+    pool_builder.build()
 }
